import unittest

from djcopybook import fixedwidth
from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests import record_helper


class ListFieldTests(unittest.TestCase):

    def test_to_python_returns_list_of_records(self):
        f = fields.ListField(record=record_helper.RecordOne, length=2)
        records = [record_helper.RecordOne(), record_helper.RecordOne()]
        python_val = f.to_python(records)
        self.assertEqual(records, python_val)
        self.assertIsInstance(python_val, list)

    def test_to_python_returns_list_of_records_when_given_string(self):
        f = fields.ListField(record=record_helper.RecordOne, length=2)

        records = f.to_python("ABCDE1111111ZYXWV9999999")
        self.assertEqual(len(records), 2)

        record_one = records[0]
        self.assertEqual("ABCDE", record_one.field_one)
        self.assertEqual(1111111, record_one.field_two)

        record_two = records[1]
        self.assertEqual("ZYXWV", record_two.field_one)
        self.assertEqual(9999999, record_two.field_two)

    def test_to_python_returns_list_of_records_when_given_unicode(self):
        f = fields.ListField(record=record_helper.RecordOne, length=2)

        records = f.to_python(u"ABCDE1111111ZYXWV9999999")
        self.assertEqual(len(records), 2)

        record_one = records[0]
        self.assertEqual("ABCDE", record_one.field_one)
        self.assertEqual(1111111, record_one.field_two)

        record_two = records[1]
        self.assertEqual("ZYXWV", record_two.field_one)
        self.assertEqual(9999999, record_two.field_two)

    def test_to_python_raises_type_error_when_not_given_record_class_instances(self):
        f = fields.ListField(record=record_helper.RecordOne, length=2)
        records = [record_helper.RecordOne(), fixedwidth.Record()]

        with self.assertRaises(TypeError) as e:
            f.to_python(records)
        self.assertEqual("List field must contain instances of 'RecordOne'.", str(e.exception))

    def test_defaults_field_to_empty_list_when_no_records_initially_given(self):
        class TestRecord(fixedwidth.Record):
            field_one = fields.ListField(record_helper.RecordOne, length=2)

        r = TestRecord()
        self.assertEqual([], r.field_one)

    def test_to_record_returns_string_of_all_records_from_list_field(self):
        class TestRecord(fixedwidth.Record):
            list_field = fields.ListField(record_helper.RecordOne, length=2)

        record_one = record_helper.RecordOne(field_one="AAAAA", field_two=1111111)
        record_two = record_helper.RecordOne(field_one="BBBBB", field_two=2222222)
        r = TestRecord(list_field=[record_one, record_two])

        self.assertEqual("AAAAA1111111BBBBB2222222", r.to_record())

    def test_to_record_pads_with_empty_records_when_not_all_records_used(self):

        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=5)
            field_two = fields.IntegerField(length=7)

        class TestRecord(fixedwidth.Record):
            list_field = fields.ListField(ListRecord, length=2)

        record_one = ListRecord(field_one="AAAAA", field_two=1111111)
        r = TestRecord(list_field=[record_one])

        self.assertEqual("AAAAA1111111     0000000", r.to_record())

    def test_check_record_length_returns_field_length_error_when_too_many_records_used(self):

        list_field = fields.ListField(record_helper.RecordOne, length=2)
        list_field.attname = "list_field"

        record_one = record_helper.RecordOne(field_one="AAAAA", field_two=1111111)
        record_two = record_helper.RecordOne(field_one="BBBBB", field_two=2222222)
        record_three = record_helper.RecordOne(field_one="CCCCC", field_two=3333333)

        records = [record_one, record_two, record_three]
        with self.assertRaises(fields.FieldLengthError) as e:
            list_field._check_record_length(''.join(r.to_record() for r in records))

        self.assertEqual("'list_field' contains 3 records but can only have 2.", str(e.exception))

    def test_populates_list_field_with_nested_dictionaries(self):
        input_data = {'threeve': [
            {'other_field': 'xyz', 'frag': {'field_one': 1, 'field_two': 2}},
            {'other_field': 'abc', 'frag': {'field_one': 3, 'field_two': 4}}
        ]}
        record = record_helper.RecordFive(**input_data)
        self.assertEqual('xyz', record.threeve[0].other_field)
        self.assertEqual('abc', record.threeve[1].other_field)
        self.assertEqual('1', record.threeve[0].frag.field_one)
        self.assertEqual('3', record.threeve[1].frag.field_one)
        self.assertEqual(2, record.threeve[0].frag.field_two)
        self.assertEqual(4, record.threeve[1].frag.field_two)



import unittest

from djcopybook.fixedwidth import fields


class StringFieldTests(unittest.TestCase):

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.StringField(length=5)
        self.assertEqual("AA   ", field.to_record("AA"))

    def test_to_record_returns_padded_blank_string_when_value_is_None(self):
        field = fields.StringField(length=5)
        self.assertEqual("     ", field.to_record(None))

    def test_to_python_turns_value_to_string(self):
        field = fields.StringField(length=5)
        python_val = field.to_python(10)
        self.assertEqual("10", python_val)
        self.assertIsInstance(python_val, str)

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.StringField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_empty_string_when_received_all_blanks(self):
        field = fields.StringField(length=5)
        self.assertEqual("", field.to_python("     "))




from unittest import TestCase
from djcopybook.fixedwidth import fields


class NullBooleanFieldTests(TestCase):

    def test_subclasses_BooleanField(self):
        self.assertTrue(issubclass(fields.NullBooleanField, fields.BooleanField))

    def test_to_record_returns_y_when_value_is_true(self):
        field = fields.NullBooleanField()
        self.assertEqual('Y', field.to_record(True))

    def test_to_record_returns_n_when_value_is_false(self):
        field = fields.NullBooleanField()
        self.assertEqual('N', field.to_record(False))

    def test_to_record_returns_empty_when_string_is_empty_space(self):
        field = fields.NullBooleanField()
        self.assertEqual(' ', field.to_record(' '))

    def test_to_record_returns_empty_when_value_is_None(self):
        field = fields.NullBooleanField()
        self.assertEqual(' ', field.to_record(None))

    def test_to_record_raises_value_error_when_data_type_invalid(self):
        field = fields.NullBooleanField()
        with self.assertRaises(ValueError) as e:
            field.to_record("My Cat")
        self.assertEqual("Value Must be Boolean or None. You gave 'My Cat'", str(e.exception))

    def test_to_python_returns_True_when_value_is_Y(self):
        field = fields.NullBooleanField()
        self.assertEqual(True, field.to_python("Y"))

    def test_to_python_returns_False_when_value_is_N(self):
        field = fields.NullBooleanField()
        self.assertEqual(False, field.to_python("N"))

    def test_to_python_returns_None_when_value_is_empty(self):
        field = fields.NullBooleanField()
        self.assertEqual(None, field.to_python(" "))

    def test_to_python_returns_value_when_already_boolean(self):
        field = fields.NullBooleanField()
        self.assertEqual(True, field.to_python(True))



import unittest

from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests import record_helper


class BooleanFieldTests(unittest.TestCase):

    def test_to_record_returns_y_when_value_is_true(self):
        field = fields.BooleanField()
        self.assertEqual('Y', field.to_record(True))

    def test_to_record_returns_false_when_string_is_n(self):
        field = fields.BooleanField()
        self.assertEqual('N', field.to_record(False))

    def test_to_record_returns_when_string_is_empty_space(self):
        field = fields.BooleanField()
        self.assertEqual('N', field.to_record(' '))

    def test_to_record_returns_python_false_when_empty_string_is_empty(self):
        field = fields.BooleanField()
        self.assertEqual('N', field.to_record(''))

    def test_to_record_will_raise_exception_when_value_is_none(self):
        field = fields.BooleanField()
        with self.assertRaises(ValueError):
            field.to_record(field.to_record())

    def test_to_record_will_raise_exception_when_value_is_not_y_or_n(self):
        field = fields.BooleanField()
        with self.assertRaises(ValueError):
            field.to_record("X")

    def test_from_record_will_return_the_correct_boolean_values(self):
        r = record_helper.RecordSix.from_record("YN ")
        self.assertEqual(True, r.first)
        self.assertEqual(False, r.second)
        self.assertEqual(False, r.third)



import unittest

from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests.record_helper import RecordOne


class FixedWidthFieldTests(unittest.TestCase):

    def get_field(self, auto_truncate=False, **kwargs):
        f = fields.FixedWidthField(**kwargs)
        f.attname = "field_one"
        f.auto_truncate = auto_truncate
        return f

    def test_raises_field_length_error_when_value_is_longer_than_allowed(self):
        f = self.get_field(length=5)
        with self.assertRaises(fields.FieldLengthError) as e:
            f.get_record_value("This is a long")

        msg = "'field_one' value 'This is a long' is longer than 5 chars."
        self.assertEqual(msg, str(e.exception))

    def test_truncates_field_length_when_has_auto_truncate_on(self):
        f = self.get_field(auto_truncate=True, length=5)
        val = f.get_record_value("This is too long")
        self.assertEqual("This ", val)

    def test_get_default_returns_none_when_no_default(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual(None, field.get_default())

    def test_get_default_returns_default_when_present(self):
        field = fields.FixedWidthField(length=5, default="ABC")
        self.assertEqual("ABC", field.get_default())

    def test_get_default_returns_called_default_when_callable(self):
        def get_default():
            return "Default Value"

        field = fields.FixedWidthField(length=15, default=get_default)
        self.assertEqual("Default Value", field.get_default())

    def test_to_record_returns_string_of_value_padded_to_length(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("ALM  ", field.to_record("ALM"))

    def test_to_record_returns_padded_blank_string_when_value_is_None(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("     ", field.to_record(None))

    def test_to_python_turns_value_to_string(self):
        field = fields.FixedWidthField(length=5)
        python_val = field.to_python(10)
        self.assertEqual("10", python_val)
        self.assertIsInstance(python_val, str)

    def test_to_python_strips_extra_padding_from_right(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual("AA", field.to_python("AA   "))

    def test_to_python_returns_none_when_value_is_none(self):
        field = fields.FixedWidthField(length=5)
        self.assertEqual(None, field.to_python(None))

    def test_returns_default_when_removed_from_record(self):
        record = RecordOne()
        record_dir = dir(record)
        for item in record_dir:
            if item.startswith('field_one'):
                field_name = item
        delattr(record, field_name)
        self.assertEqual('AA', record.field_one)



import unittest

from djcopybook.fixedwidth import fields


class NewLineFieldTests(unittest.TestCase):

    def test_to_record_returns_new_line_character(self):
        field = fields.NewLineField()
        self.assertEqual("\n", field.to_record("\n"))

    def test_to_python_turns_value_to_string(self):
        field = fields.NewLineField()
        python_val = field.to_python("\n")
        self.assertEqual("\n", python_val)
        self.assertIsInstance(python_val, str)



import unittest

from djcopybook import fixedwidth
from djcopybook.fixedwidth import fields
from djcopybook.fixedwidth.tests import record_helper


class RecordTests(unittest.TestCase):

    def test_keeps_list_of_fields_in_order_when_record_is_instantiated(self):
        r = record_helper.RecordOne()
        self.assertEqual(['field_one', 'field_two'], list(r.fields.keys()))
        self.assertEqual(['field_one', 'field_two'], list(r.base_fields.keys()))

    def test_sets_auto_truncate_field_on_fields_when_instantiated(self):
        r = record_helper.RecordOne()

        self.assertEqual(2, len(r.fields))
        for field in r.fields.values():
            self.assertEqual(False, field.auto_truncate)

    def test_sets_auto_truncate_field_on_fields_when_turned_on(self):
        r = record_helper.RecordTwo()

        self.assertEqual(4, len(r.fields))
        for field in r.fields.values():
            self.assertEqual(True, field.auto_truncate)

    def test_record_inheritance_maintains_order(self):
        r = record_helper.RecordTwo()
        self.assertEqual(['field_one', 'field_two', 'field_three', 'field_four'], list(r.fields.keys()))
        self.assertEqual(['field_one', 'field_two', 'field_three', 'field_four'], list(r.base_fields.keys()))

    def test_fills_attributes_with_values_given_in_init(self):
        r = record_helper.RecordOne(field_one=10, field_two=5)
        self.assertEqual("10", r.field_one)
        self.assertEqual(5, r.field_two)

    def test_to_record_returns_record_for_each_field(self):
        r = record_helper.RecordOne(field_one="test", field_two=500)
        self.assertEqual("test 0000500", r.to_record())

    def test_returns_field_default_value_when_not_given(self):
        r = record_helper.RecordOne(field_two=10)
        self.assertEqual("AA", r.field_one)

    def test_ignores_kwarg_value_when_field_not_declared_on_record(self):
        r = record_helper.RecordOne(field_one="abc", no_field="test")
        self.assertEqual("abc", r.field_one)
        self.assertFalse(hasattr(r, "no_field"))

    def test_len_on_record_returns_total_fixed_width_length(self):
        r = record_helper.RecordOne()
        self.assertEqual(12, len(r))

    def test_len_on_record_returns_total_amount_with_list_field(self):
        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=1)
            field_two = fields.IntegerField(length=2)

        class TestRecord(fixedwidth.Record):
            name = fields.StringField(length=2)
            list_field = fields.ListField(ListRecord, length=2)
            end = fields.StringField(length=1)

        # should get same len whether using instance or just the class
        r = TestRecord()
        self.assertEqual(9, len(r))
        self.assertEqual(9, len(TestRecord))

    def test_str_on_record_returns_to_record_value(self):
        r = record_helper.RecordOne(field_one="test", field_two=500)
        self.assertEqual("test 0000500", str(r))

    def test_from_record_turns_fixed_width_string_to_record_object(self):
        r = record_helper.RecordOne.from_record("test 0000500")
        self.assertEqual("test", r.field_one)
        self.assertEqual(500, r.field_two)

    def test_from_record_turns_fixed_width_string_to_record_with_list_field(self):
        class ListRecord(fixedwidth.Record):
            field_one = fields.StringField(length=1)
            field_two = fields.IntegerField(length=2)

        class TestRecord(fixedwidth.Record):
            name = fields.StringField(length=2)
            list_field = fields.ListField(ListRecord, length=2)
            end = fields.StringField(length=1)

        r = TestRecord.from_record("XXA05B10Z")
        self.assertEqual("XX", r.name)
        self.assertEqual("Z", r.end)

        list_field_one = r.list_field[0]
        self.assertEqual("A", list_field_one.field_one)
        self.assertEqual(5, list_field_one.field_two)

        list_field_two = r.list_field[1]
        self.assertEqual("B", list_field_two.field_one)
        self.assertEqual(10, list_field_two.field_two)

        self.assertEqual("XXA05B10Z", r.to_record())

    def test_from_record_raises_value_error_when_fixed_width_string_less_than_record_length(self):
        with self.assertRaises(ValueError) as e:
            record_helper.RecordOne.from_record("test 000050")
        self.assertEqual("Fixed width record length is 11 but should be 12.", str(e.exception))

    def test_from_record_raises_value_error_when_fixed_width_string_greater_than_record_length(self):
        with self.assertRaises(ValueError) as e:
            record_helper.RecordOne.from_record("test 00005000")
        self.assertEqual("Fixed width record length is 13 but should be 12.", str(e.exception))

    def test_truncates_each_field_when_auto_truncate(self):

        class TruncRecord(fixedwidth.Record):
            auto_truncate = True

            char = fields.StringField(length=2)
            integer = fields.IntegerField(length=3)

        r = TruncRecord(char="Too long", integer=12345)
        record = r.to_record()
        self.assertEqual("To123", record)




from decimal import Decimal
from unittest import TestCase
from djcopybook.fixedwidth import fields


class ImpliedDecimalFieldTests(TestCase):

    def test_subclasses_decimal_field(self):
        self.assertTrue(issubclass(fields.ImpliedDecimalField, fields.DecimalField))

    def test_pads_number_when_zero_decimal(self):
        f = fields.ImpliedDecimalField(length=5, decimals=0)
        self.assertEqual('00300', f.to_record(300))

    def test_pads_number_when_two_decimals(self):
        f = fields.ImpliedDecimalField(length=7, decimals=2)
        self.assertEqual('0030025', f.to_record(300.25))

    def test_truncates_number_when_too_many_decimals(self):
        f = fields.ImpliedDecimalField(length=12, decimals=2)
        self.assertEqual('000003050025', f.to_record(30500.25325))

    def test_pads_decimal_number_type_appropriately(self):
        f = fields.ImpliedDecimalField(length=10, decimals=2)
        self.assertEqual('0007500075', f.to_record(Decimal("75000.75")))

    def test_value_of_none_is_all_zeros(self):
        f = fields.ImpliedDecimalField(length=8, decimals=2)
        self.assertEqual('00000000', f.to_record(None))

    def test_to_python_returns_decimal_type(self):
        f = fields.ImpliedDecimalField(length=10, decimals=2)
        self.assertEqual(Decimal("7500.33"), f.to_python("0000750033"))

    def test_to_python_returns_decimal_of_value_when_already_not_a_string(self):
        f = fields.ImpliedDecimalField(length=10, decimals=2)
        self.assertEqual(Decimal("750.33"), f.to_python(750.33))

    def test_to_python_returns_decimal_type_when_no_decimals_declared(self):
        f = fields.ImpliedDecimalField(length=10, decimals=0)
        self.assertEqual(Decimal("750033"), f.to_python("000750033"))

    def test_to_python_returns_None_when_value_is_None(self):
        f = fields.ImpliedDecimalField(length=10, decimals=2)
        self.assertEqual(None, f.to_python(None))

    def test_to_python_returns_None_when_value_is_empty_string(self):
        f = fields.ImpliedDecimalField(length=10, decimals=2)
        self.assertEqual(None, f.to_python("          "))



from datetime import date, datetime
import unittest

from djcopybook import fixedwidth
from djcopybook.fixedwidth import fields


class DateFieldTests(unittest.TestCase):

    def test_to_record_returns_formatted_date(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("20110831", field.to_record(date(2011, 8, 31)))

    def test_to_record_returns_padded_spaces_when_value_is_None(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("        ", field.to_record(None))

    def test_to_record_returns_padded_spaces_when_value_is_empty_string(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual("        ", field.to_record(''))

    def test_to_python_turns_string_to_date_field(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = date(2011, 8, 31)
        python_val = field.to_python("20110831")
        self.assertEqual(expected_date, python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_returns_none_when_no_value_given(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual(None, field.to_python(None))

    def test_to_python_returns_none_when_empty_string_given(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        self.assertEqual(None, field.to_python("        "))

    def test_to_python_returns_date_when_already_date(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = date(2011, 8, 31)
        python_val = field.to_python(expected_date)
        self.assertEqual(expected_date, python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_returns_date_when_given_datetime(self):
        field = fields.DateField(length=8, format="%Y%m%d")
        expected_date = datetime(2011, 8, 31)
        python_val = field.to_python(expected_date)
        self.assertEqual(expected_date.date(), python_val)
        self.assertIsInstance(python_val, date)

    def test_to_python_raises_value_error_when_value_given_has_characters(self):
        field = fields.DateField(length=5)
        with self.assertRaises(ValueError):
            self.assertEqual(None, field.to_python("0001A"))

    def test_datefield_should_allow_string_default_date(self):
        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default="08/01/2001")

        c = TestRecord()
        self.assertEqual(date(2001, 8, 1), c.field_one)

    def test_datefield_should_allow_date_default_date(self):
        default_date = date(2001, 1, 1)

        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default=default_date)

        c = TestRecord()
        self.assertEqual(date(2001, 1, 1), c.field_one)

    def test_datefield_should_allow_callable_date(self):
        def get_date():
            return date(2000, 1, 1)

        class TestRecord(fixedwidth.Record):
            field_one = fields.DateField(length=10, format="%m/%d/%Y", default=get_date)

        c = TestRecord()
        self.assertEqual(date(2000, 1, 1), c.field_one)
